//
// Copyright (c) 2017 SWET. All rights reserved.
//

import UIKit

private let minPasswordLength = 6
private let emailRegex = "[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,}"

protocol ValidationRule {

    /// UITextFieldを検証する
    ///
    /// - Parameter textField: UITextField
    /// - Returns: ValidationResult 検証結果
    func validate(textField: UITextField) -> ValidationResult
}

/// Emailの検証ルール
struct EmailValidationRule: ValidationRule {

    func validate(textField: UITextField) -> ValidationResult {
        
        guard let text = textField.text, !text.isEmpty else {
            return .invalid([FieldError(field: textField, messageKey: "error_required")])
        }

        if !NSPredicate(format:"SELF MATCHES %@", emailRegex).evaluate(with: text) {
            return .invalid([FieldError(field: textField, messageKey: "error_invalid_email")])
        } else {
            return .valid
        }
    }

}

/// パスワードの検証ルール
struct PasswordValidationRule: ValidationRule {

    func validate(textField: UITextField) -> ValidationResult {
        
        guard let text = textField.text, !text.isEmpty else {
            return .invalid([FieldError(field: textField, messageKey: "error_required")])
        }

        if text.characters.count < minPasswordLength {
            return .invalid([FieldError(field: textField, messageKey: "error_invalid_password")])
        } else {
            return .valid
        }
    }

}

/// 空文字NGの検証ルール
class NoEmptyValidationRule: ValidationRule {

    func validate(textField: UITextField) -> ValidationResult {

        guard let text = textField.text, !text.isEmpty else {
            return .invalid([FieldError(field: textField, messageKey: "error_required")])
        }
        
        return .valid
    }

}
